# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseDashboardExperimentFilterValues(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'bool_result': 'bool',
        'datetime_result': 'float',
        'duration_result': 'OrmDurationValue',
        'enum_results': 'list[str]',
        'number_results': 'list[float]',
        'object_results': 'list[ResponseFieldObjectValue]',
        'string_results': 'list[str]',
        'tag_results': 'list[ResponseTagResponse]'
    }

    attribute_map = {
        'bool_result': 'bool_result',
        'datetime_result': 'datetime_result',
        'duration_result': 'duration_result',
        'enum_results': 'enum_results',
        'number_results': 'number_results',
        'object_results': 'object_results',
        'string_results': 'string_results',
        'tag_results': 'tag_results'
    }

    def __init__(self, bool_result=None, datetime_result=None, duration_result=None, enum_results=None, number_results=None, object_results=None, string_results=None, tag_results=None, local_vars_configuration=None):  # noqa: E501
        """ResponseDashboardExperimentFilterValues - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._bool_result = None
        self._datetime_result = None
        self._duration_result = None
        self._enum_results = None
        self._number_results = None
        self._object_results = None
        self._string_results = None
        self._tag_results = None
        self.discriminator = None

        self.bool_result = bool_result
        self.datetime_result = datetime_result
        if duration_result is not None:
            self.duration_result = duration_result
        self.enum_results = enum_results
        self.number_results = number_results
        self.object_results = object_results
        self.string_results = string_results
        self.tag_results = tag_results

    @property
    def bool_result(self):
        """Gets the bool_result of this ResponseDashboardExperimentFilterValues.  # noqa: E501


        :return: The bool_result of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :rtype: bool
        """
        return self._bool_result

    @bool_result.setter
    def bool_result(self, bool_result):
        """Sets the bool_result of this ResponseDashboardExperimentFilterValues.


        :param bool_result: The bool_result of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :type bool_result: bool
        """

        self._bool_result = bool_result

    @property
    def datetime_result(self):
        """Gets the datetime_result of this ResponseDashboardExperimentFilterValues.  # noqa: E501


        :return: The datetime_result of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :rtype: float
        """
        return self._datetime_result

    @datetime_result.setter
    def datetime_result(self, datetime_result):
        """Sets the datetime_result of this ResponseDashboardExperimentFilterValues.


        :param datetime_result: The datetime_result of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :type datetime_result: float
        """

        self._datetime_result = datetime_result

    @property
    def duration_result(self):
        """Gets the duration_result of this ResponseDashboardExperimentFilterValues.  # noqa: E501


        :return: The duration_result of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :rtype: OrmDurationValue
        """
        return self._duration_result

    @duration_result.setter
    def duration_result(self, duration_result):
        """Sets the duration_result of this ResponseDashboardExperimentFilterValues.


        :param duration_result: The duration_result of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :type duration_result: OrmDurationValue
        """

        self._duration_result = duration_result

    @property
    def enum_results(self):
        """Gets the enum_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501


        :return: The enum_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :rtype: list[str]
        """
        return self._enum_results

    @enum_results.setter
    def enum_results(self, enum_results):
        """Sets the enum_results of this ResponseDashboardExperimentFilterValues.


        :param enum_results: The enum_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :type enum_results: list[str]
        """
        if self.local_vars_configuration.client_side_validation and enum_results is None:  # noqa: E501
            raise ValueError("Invalid value for `enum_results`, must not be `None`")  # noqa: E501

        self._enum_results = enum_results

    @property
    def number_results(self):
        """Gets the number_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501


        :return: The number_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :rtype: list[float]
        """
        return self._number_results

    @number_results.setter
    def number_results(self, number_results):
        """Sets the number_results of this ResponseDashboardExperimentFilterValues.


        :param number_results: The number_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :type number_results: list[float]
        """
        if self.local_vars_configuration.client_side_validation and number_results is None:  # noqa: E501
            raise ValueError("Invalid value for `number_results`, must not be `None`")  # noqa: E501

        self._number_results = number_results

    @property
    def object_results(self):
        """Gets the object_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501


        :return: The object_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :rtype: list[ResponseFieldObjectValue]
        """
        return self._object_results

    @object_results.setter
    def object_results(self, object_results):
        """Sets the object_results of this ResponseDashboardExperimentFilterValues.


        :param object_results: The object_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :type object_results: list[ResponseFieldObjectValue]
        """
        if self.local_vars_configuration.client_side_validation and object_results is None:  # noqa: E501
            raise ValueError("Invalid value for `object_results`, must not be `None`")  # noqa: E501

        self._object_results = object_results

    @property
    def string_results(self):
        """Gets the string_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501


        :return: The string_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :rtype: list[str]
        """
        return self._string_results

    @string_results.setter
    def string_results(self, string_results):
        """Sets the string_results of this ResponseDashboardExperimentFilterValues.


        :param string_results: The string_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :type string_results: list[str]
        """
        if self.local_vars_configuration.client_side_validation and string_results is None:  # noqa: E501
            raise ValueError("Invalid value for `string_results`, must not be `None`")  # noqa: E501

        self._string_results = string_results

    @property
    def tag_results(self):
        """Gets the tag_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501


        :return: The tag_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :rtype: list[ResponseTagResponse]
        """
        return self._tag_results

    @tag_results.setter
    def tag_results(self, tag_results):
        """Sets the tag_results of this ResponseDashboardExperimentFilterValues.


        :param tag_results: The tag_results of this ResponseDashboardExperimentFilterValues.  # noqa: E501
        :type tag_results: list[ResponseTagResponse]
        """
        if self.local_vars_configuration.client_side_validation and tag_results is None:  # noqa: E501
            raise ValueError("Invalid value for `tag_results`, must not be `None`")  # noqa: E501

        self._tag_results = tag_results

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseDashboardExperimentFilterValues):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseDashboardExperimentFilterValues):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponsePipelineStepExecution(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'context': 'V1Context',
        'created_dt': 'datetime',
        'description': 'str',
        'exposed_services': 'list[ResponseExposedService]',
        'external_service_result': 'ResponsePipelineStepExternalServiceResult',
        'fail_result': 'object',
        'id': 'int',
        'if_result': 'ResponsePipelineStepIfResult',
        'input_result': 'ResponsePipelineStepManualInputResult',
        'judgment_result': 'ResponsePipelineStepManualJudgmentResult',
        'logs': 'list[InfluxdbWorkloadLog]',
        'notification_result': 'ResponsePipelineStepNotificationResult',
        'run_result': 'ResponsePipelineStepRunResult',
        'status': 'str',
        'step_spec_id': 'int',
        'title': 'str',
        'type': 'str'
    }

    attribute_map = {
        'context': 'context',
        'created_dt': 'created_dt',
        'description': 'description',
        'exposed_services': 'exposed_services',
        'external_service_result': 'external_service_result',
        'fail_result': 'fail_result',
        'id': 'id',
        'if_result': 'if_result',
        'input_result': 'input_result',
        'judgment_result': 'judgment_result',
        'logs': 'logs',
        'notification_result': 'notification_result',
        'run_result': 'run_result',
        'status': 'status',
        'step_spec_id': 'step_spec_id',
        'title': 'title',
        'type': 'type'
    }

    def __init__(self, context=None, created_dt=None, description=None, exposed_services=None, external_service_result=None, fail_result=None, id=None, if_result=None, input_result=None, judgment_result=None, logs=None, notification_result=None, run_result=None, status=None, step_spec_id=None, title=None, type=None, local_vars_configuration=None):  # noqa: E501
        """ResponsePipelineStepExecution - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._context = None
        self._created_dt = None
        self._description = None
        self._exposed_services = None
        self._external_service_result = None
        self._fail_result = None
        self._id = None
        self._if_result = None
        self._input_result = None
        self._judgment_result = None
        self._logs = None
        self._notification_result = None
        self._run_result = None
        self._status = None
        self._step_spec_id = None
        self._title = None
        self._type = None
        self.discriminator = None

        if context is not None:
            self.context = context
        if created_dt is not None:
            self.created_dt = created_dt
        if description is not None:
            self.description = description
        if exposed_services is not None:
            self.exposed_services = exposed_services
        if external_service_result is not None:
            self.external_service_result = external_service_result
        if fail_result is not None:
            self.fail_result = fail_result
        self.id = id
        if if_result is not None:
            self.if_result = if_result
        if input_result is not None:
            self.input_result = input_result
        if judgment_result is not None:
            self.judgment_result = judgment_result
        self.logs = logs
        if notification_result is not None:
            self.notification_result = notification_result
        if run_result is not None:
            self.run_result = run_result
        if status is not None:
            self.status = status
        self.step_spec_id = step_spec_id
        self.title = title
        self.type = type

    @property
    def context(self):
        """Gets the context of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The context of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: V1Context
        """
        return self._context

    @context.setter
    def context(self, context):
        """Sets the context of this ResponsePipelineStepExecution.


        :param context: The context of this ResponsePipelineStepExecution.  # noqa: E501
        :type context: V1Context
        """

        self._context = context

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The created_dt of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponsePipelineStepExecution.


        :param created_dt: The created_dt of this ResponsePipelineStepExecution.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def description(self):
        """Gets the description of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The description of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ResponsePipelineStepExecution.


        :param description: The description of this ResponsePipelineStepExecution.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def exposed_services(self):
        """Gets the exposed_services of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The exposed_services of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: list[ResponseExposedService]
        """
        return self._exposed_services

    @exposed_services.setter
    def exposed_services(self, exposed_services):
        """Sets the exposed_services of this ResponsePipelineStepExecution.


        :param exposed_services: The exposed_services of this ResponsePipelineStepExecution.  # noqa: E501
        :type exposed_services: list[ResponseExposedService]
        """

        self._exposed_services = exposed_services

    @property
    def external_service_result(self):
        """Gets the external_service_result of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The external_service_result of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: ResponsePipelineStepExternalServiceResult
        """
        return self._external_service_result

    @external_service_result.setter
    def external_service_result(self, external_service_result):
        """Sets the external_service_result of this ResponsePipelineStepExecution.


        :param external_service_result: The external_service_result of this ResponsePipelineStepExecution.  # noqa: E501
        :type external_service_result: ResponsePipelineStepExternalServiceResult
        """

        self._external_service_result = external_service_result

    @property
    def fail_result(self):
        """Gets the fail_result of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The fail_result of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: object
        """
        return self._fail_result

    @fail_result.setter
    def fail_result(self, fail_result):
        """Sets the fail_result of this ResponsePipelineStepExecution.


        :param fail_result: The fail_result of this ResponsePipelineStepExecution.  # noqa: E501
        :type fail_result: object
        """

        self._fail_result = fail_result

    @property
    def id(self):
        """Gets the id of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The id of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponsePipelineStepExecution.


        :param id: The id of this ResponsePipelineStepExecution.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def if_result(self):
        """Gets the if_result of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The if_result of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: ResponsePipelineStepIfResult
        """
        return self._if_result

    @if_result.setter
    def if_result(self, if_result):
        """Sets the if_result of this ResponsePipelineStepExecution.


        :param if_result: The if_result of this ResponsePipelineStepExecution.  # noqa: E501
        :type if_result: ResponsePipelineStepIfResult
        """

        self._if_result = if_result

    @property
    def input_result(self):
        """Gets the input_result of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The input_result of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: ResponsePipelineStepManualInputResult
        """
        return self._input_result

    @input_result.setter
    def input_result(self, input_result):
        """Sets the input_result of this ResponsePipelineStepExecution.


        :param input_result: The input_result of this ResponsePipelineStepExecution.  # noqa: E501
        :type input_result: ResponsePipelineStepManualInputResult
        """

        self._input_result = input_result

    @property
    def judgment_result(self):
        """Gets the judgment_result of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The judgment_result of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: ResponsePipelineStepManualJudgmentResult
        """
        return self._judgment_result

    @judgment_result.setter
    def judgment_result(self, judgment_result):
        """Sets the judgment_result of this ResponsePipelineStepExecution.


        :param judgment_result: The judgment_result of this ResponsePipelineStepExecution.  # noqa: E501
        :type judgment_result: ResponsePipelineStepManualJudgmentResult
        """

        self._judgment_result = judgment_result

    @property
    def logs(self):
        """Gets the logs of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The logs of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: list[InfluxdbWorkloadLog]
        """
        return self._logs

    @logs.setter
    def logs(self, logs):
        """Sets the logs of this ResponsePipelineStepExecution.


        :param logs: The logs of this ResponsePipelineStepExecution.  # noqa: E501
        :type logs: list[InfluxdbWorkloadLog]
        """
        if self.local_vars_configuration.client_side_validation and logs is None:  # noqa: E501
            raise ValueError("Invalid value for `logs`, must not be `None`")  # noqa: E501

        self._logs = logs

    @property
    def notification_result(self):
        """Gets the notification_result of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The notification_result of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: ResponsePipelineStepNotificationResult
        """
        return self._notification_result

    @notification_result.setter
    def notification_result(self, notification_result):
        """Sets the notification_result of this ResponsePipelineStepExecution.


        :param notification_result: The notification_result of this ResponsePipelineStepExecution.  # noqa: E501
        :type notification_result: ResponsePipelineStepNotificationResult
        """

        self._notification_result = notification_result

    @property
    def run_result(self):
        """Gets the run_result of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The run_result of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: ResponsePipelineStepRunResult
        """
        return self._run_result

    @run_result.setter
    def run_result(self, run_result):
        """Sets the run_result of this ResponsePipelineStepExecution.


        :param run_result: The run_result of this ResponsePipelineStepExecution.  # noqa: E501
        :type run_result: ResponsePipelineStepRunResult
        """

        self._run_result = run_result

    @property
    def status(self):
        """Gets the status of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The status of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponsePipelineStepExecution.


        :param status: The status of this ResponsePipelineStepExecution.  # noqa: E501
        :type status: str
        """

        self._status = status

    @property
    def step_spec_id(self):
        """Gets the step_spec_id of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The step_spec_id of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: int
        """
        return self._step_spec_id

    @step_spec_id.setter
    def step_spec_id(self, step_spec_id):
        """Sets the step_spec_id of this ResponsePipelineStepExecution.


        :param step_spec_id: The step_spec_id of this ResponsePipelineStepExecution.  # noqa: E501
        :type step_spec_id: int
        """
        if self.local_vars_configuration.client_side_validation and step_spec_id is None:  # noqa: E501
            raise ValueError("Invalid value for `step_spec_id`, must not be `None`")  # noqa: E501

        self._step_spec_id = step_spec_id

    @property
    def title(self):
        """Gets the title of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The title of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this ResponsePipelineStepExecution.


        :param title: The title of this ResponsePipelineStepExecution.  # noqa: E501
        :type title: str
        """
        if self.local_vars_configuration.client_side_validation and title is None:  # noqa: E501
            raise ValueError("Invalid value for `title`, must not be `None`")  # noqa: E501

        self._title = title

    @property
    def type(self):
        """Gets the type of this ResponsePipelineStepExecution.  # noqa: E501


        :return: The type of this ResponsePipelineStepExecution.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ResponsePipelineStepExecution.


        :param type: The type of this ResponsePipelineStepExecution.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponsePipelineStepExecution):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponsePipelineStepExecution):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseSweepExperimentInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'experiment_id': 'int',
        'experiment_number': 'int',
        'initialized_dt': 'datetime',
        'objective_metric_name': 'str',
        'objective_metric_value': 'float',
        'parameters': 'list[ResponseParameterInfo]'
    }

    attribute_map = {
        'experiment_id': 'experiment_id',
        'experiment_number': 'experiment_number',
        'initialized_dt': 'initialized_dt',
        'objective_metric_name': 'objective_metric_name',
        'objective_metric_value': 'objective_metric_value',
        'parameters': 'parameters'
    }

    def __init__(self, experiment_id=None, experiment_number=None, initialized_dt=None, objective_metric_name=None, objective_metric_value=None, parameters=None, local_vars_configuration=None):  # noqa: E501
        """ResponseSweepExperimentInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._experiment_id = None
        self._experiment_number = None
        self._initialized_dt = None
        self._objective_metric_name = None
        self._objective_metric_value = None
        self._parameters = None
        self.discriminator = None

        if experiment_id is not None:
            self.experiment_id = experiment_id
        if experiment_number is not None:
            self.experiment_number = experiment_number
        self.initialized_dt = initialized_dt
        if objective_metric_name is not None:
            self.objective_metric_name = objective_metric_name
        self.objective_metric_value = objective_metric_value
        if parameters is not None:
            self.parameters = parameters

    @property
    def experiment_id(self):
        """Gets the experiment_id of this ResponseSweepExperimentInfo.  # noqa: E501


        :return: The experiment_id of this ResponseSweepExperimentInfo.  # noqa: E501
        :rtype: int
        """
        return self._experiment_id

    @experiment_id.setter
    def experiment_id(self, experiment_id):
        """Sets the experiment_id of this ResponseSweepExperimentInfo.


        :param experiment_id: The experiment_id of this ResponseSweepExperimentInfo.  # noqa: E501
        :type experiment_id: int
        """

        self._experiment_id = experiment_id

    @property
    def experiment_number(self):
        """Gets the experiment_number of this ResponseSweepExperimentInfo.  # noqa: E501


        :return: The experiment_number of this ResponseSweepExperimentInfo.  # noqa: E501
        :rtype: int
        """
        return self._experiment_number

    @experiment_number.setter
    def experiment_number(self, experiment_number):
        """Sets the experiment_number of this ResponseSweepExperimentInfo.


        :param experiment_number: The experiment_number of this ResponseSweepExperimentInfo.  # noqa: E501
        :type experiment_number: int
        """

        self._experiment_number = experiment_number

    @property
    def initialized_dt(self):
        """Gets the initialized_dt of this ResponseSweepExperimentInfo.  # noqa: E501


        :return: The initialized_dt of this ResponseSweepExperimentInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._initialized_dt

    @initialized_dt.setter
    def initialized_dt(self, initialized_dt):
        """Sets the initialized_dt of this ResponseSweepExperimentInfo.


        :param initialized_dt: The initialized_dt of this ResponseSweepExperimentInfo.  # noqa: E501
        :type initialized_dt: datetime
        """

        self._initialized_dt = initialized_dt

    @property
    def objective_metric_name(self):
        """Gets the objective_metric_name of this ResponseSweepExperimentInfo.  # noqa: E501


        :return: The objective_metric_name of this ResponseSweepExperimentInfo.  # noqa: E501
        :rtype: str
        """
        return self._objective_metric_name

    @objective_metric_name.setter
    def objective_metric_name(self, objective_metric_name):
        """Sets the objective_metric_name of this ResponseSweepExperimentInfo.


        :param objective_metric_name: The objective_metric_name of this ResponseSweepExperimentInfo.  # noqa: E501
        :type objective_metric_name: str
        """

        self._objective_metric_name = objective_metric_name

    @property
    def objective_metric_value(self):
        """Gets the objective_metric_value of this ResponseSweepExperimentInfo.  # noqa: E501


        :return: The objective_metric_value of this ResponseSweepExperimentInfo.  # noqa: E501
        :rtype: float
        """
        return self._objective_metric_value

    @objective_metric_value.setter
    def objective_metric_value(self, objective_metric_value):
        """Sets the objective_metric_value of this ResponseSweepExperimentInfo.


        :param objective_metric_value: The objective_metric_value of this ResponseSweepExperimentInfo.  # noqa: E501
        :type objective_metric_value: float
        """

        self._objective_metric_value = objective_metric_value

    @property
    def parameters(self):
        """Gets the parameters of this ResponseSweepExperimentInfo.  # noqa: E501


        :return: The parameters of this ResponseSweepExperimentInfo.  # noqa: E501
        :rtype: list[ResponseParameterInfo]
        """
        return self._parameters

    @parameters.setter
    def parameters(self, parameters):
        """Sets the parameters of this ResponseSweepExperimentInfo.


        :param parameters: The parameters of this ResponseSweepExperimentInfo.  # noqa: E501
        :type parameters: list[ResponseParameterInfo]
        """

        self._parameters = parameters

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseSweepExperimentInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseSweepExperimentInfo):
            return True

        return self.to_dict() != other.to_dict()



import sys
from typing import Optional, TextIO

import click
from ruamel.yaml import YAML

from openapi_client import ResponseModelServiceInfo
from vessl import vessl_api
from vessl.cli._base import VesslGroup
from vessl.serving import (
    create_revision_from_yaml,
    read_gateway,
    update_gateway_for_revision,
    update_gateway_from_yaml,
)
from vessl.util.exception import VesslApiException

from .util import list_http_ports, print_gateway, print_revision, validate_port_choice

cli = VesslGroup("serve")


def serving_name_callback(
    ctx: click.Context, param: click.Parameter, serving_name: Optional[str]
) -> str:
    if vessl_api.organization is None:
        vessl_api.set_organization()

    try:
        serving: ResponseModelServiceInfo = vessl_api.model_service_read_api(
            serving_name, vessl_api.organization.name
        )
    except VesslApiException as e:
        print(f"Invalid serving {serving_name}: {e.message}")
        sys.exit(1)

    ctx.meta["serving"] = serving


serving_name_option = click.option(
    "--serving",
    "serving_name",
    type=click.STRING,
    required=True,
    callback=serving_name_callback,
    expose_value=False,
    help="Name of serving.",
)

update_gateway_option = click.option(
    "--update-gateway/--no-update-gateway",
    "-g/-G",
    is_flag=True,
    default=False,
    help="Whether to update gateway so that it points to this revision.",
)

enable_gateway_if_off_option = click.option(
    "--enable-gateway-if-off/--no-enable-gateway-if-off",
    "-e/-E",
    is_flag=True,
    default=False,
    help="When updating gateway, whether to enable the gateway if it is currently off.",
)

update_gateway_weight_option = click.option(
    "--update-gateway-weight",
    type=click.INT,
    required=False,
    help=(
        "When updating gateway, the amount of traffic that should be "
        "directed to this revision (in percentage)."
    ),
)

update_gateway_port_option = click.option(
    "--update-gateway-port",
    type=click.INT,
    required=False,
    help=(
        "When updating gateway, the port to receive the traffic; "
        "this port must be defined in serving spec."
    ),
)


@cli.vessl_command(name="revision")
@click.pass_context
@serving_name_option
@click.option(
    "-f",
    "--file",
    type=click.File("r"),
    required=True,
    help="Path to YAML file for serving revision definition.",
)
@update_gateway_option
@enable_gateway_if_off_option
@update_gateway_weight_option
@update_gateway_port_option
def import_revision(
    ctx: click.Context,
    file: TextIO,
    update_gateway: bool,
    enable_gateway_if_off: bool,
    update_gateway_weight: Optional[int] = None,
    update_gateway_port: Optional[int] = None,
):
    if not update_gateway and (
        update_gateway_weight is not None or update_gateway_port is not None
    ):
        print("Cannot specify traffic weight or port when not updating gateway.")
        sys.exit(1)

    serving: ResponseModelServiceInfo = ctx.meta["serving"]

    yaml_body = file.read()
    yaml_obj = YAML().safe_load(yaml_body)

    if update_gateway:
        # Do as much validation as possible before actually creating revision.
        # weight check
        if update_gateway_weight is None:
            update_gateway_weight = 100
        elif not 1 <= update_gateway_weight <= 100:
            print(f"Invalid weight: {update_gateway_weight}% (must be between 1 and 100)")
            sys.exit(1)

        # port check
        if update_gateway_port is None:
            http_ports = list_http_ports(yaml_obj)
            if len(http_ports) != 1:
                print(
                    "Error: port for gateway was not specified, and could not automatically "
                    "determine which port to use.\n"
                    f"{len(http_ports)} port(s) was found: " + ", ".join(map(str, http_ports))
                )
                sys.exit(1)
            update_gateway_port = http_ports[0]
            print(f"Automatically choosing port {update_gateway_port} for gateway.")
        elif not 1 <= update_gateway_port <= 65535:
            print(f"Invalid port: {update_gateway_port}")
            sys.exit(1)
        else:
            validate_port_choice(yaml_obj, update_gateway_port)

        # gateway status check
        if not enable_gateway_if_off:
            gateway_current = read_gateway(
                organization=vessl_api.organization.name, serving_name=serving.name
            )
            if not gateway_current.enabled:
                print("Cannot update gateway because it is not enabled. Please enable it first.")
                print("NOTE (current status of gateway):")
                print_gateway(gateway_current)
                sys.exit(1)

    revision = create_revision_from_yaml(
        organization=vessl_api.organization.name, serving_name=serving.name, yaml_body=yaml_body
    )
    print(f"Successfully created revision in serving {serving.name}.\n")
    print_revision(revision)

    if update_gateway:
        gateway_updated = update_gateway_for_revision(
            vessl_api.organization.name,
            serving_name=serving.name,
            revision_number=revision.number,
            port=update_gateway_port,
            weight=update_gateway_weight,
        )
        print(f"Successfully updated gateway for revision #{revision.number}.\n")
        print_gateway(gateway_updated)
    else:
        print(
            "NOTE: Since --update-gateway option was not given, "
            "you cannot currently access this revision via gateway.\n\n"
            "Either use --update-gateway when creating revision, or update gateway manually."
        )


@cli.vessl_command(name="gateway")
@click.pass_context
@serving_name_option
@click.option(
    "-f",
    "--file",
    type=click.File("r"),
    required=True,
    help="Path to YAML file for serving revision definition.",
)
def import_gateway(ctx: click.Context, file: TextIO):
    serving: ResponseModelServiceInfo = ctx.meta["serving"]
    yaml_body = file.read()

    gateway = update_gateway_from_yaml(
        organization=vessl_api.organization.name, serving_name=serving.name, yaml_body=yaml_body
    )
    print(f"Successfully update gateway of serving {serving.name}.\n")
    print_gateway(gateway)



from typing import List

from openapi_client.models import ModelServiceGatewayUpdateAPIInput
from openapi_client.models import (
    OrmModelServiceGatewayTrafficSplitEntry as TrafficSplitEntry,
)
from openapi_client.models import (
    ResponseModelServiceGatewayInfo,
    ResponseModelServiceInfo,
    ResponseModelServiceRevision,
    ServingGatewayYamlImportAPIInput,
    ServingRevisionYamlImportAPIInput,
)
from vessl import vessl_api


def create_revision_from_yaml(
    organization: str, serving_name: str, yaml_body: str
) -> ResponseModelServiceRevision:
    payload = ServingRevisionYamlImportAPIInput(yaml_body)

    return vessl_api.serving_revision_yaml_import_api(
        organization_name=organization,
        model_service_name=serving_name,
        serving_revision_yaml_import_api_input=payload,
    )


def read_revision(organization: str, serving_name: str) -> ResponseModelServiceRevision:
    return vessl_api.model_service_revision_read_api(
        organization_name=organization, model_service_name=serving_name
    )


def read_gateway(organization: str, serving_name: str) -> ResponseModelServiceGatewayInfo:
    model_service: ResponseModelServiceInfo = vessl_api.model_service_read_api(
        model_service_name=serving_name,
        organization_name=organization,
    )
    return model_service.gateway_config


def update_gateway(
    organization: str, serving_name: str, gateway: ModelServiceGatewayUpdateAPIInput
) -> ResponseModelServiceGatewayInfo:
    return vessl_api.model_service_gateway_update_api(
        model_service_name=serving_name,
        organization_name=organization,
        model_service_gateway_update_api_input=gateway,
    )


def _get_updated_traffic_split_rule(
    rules_current: List[TrafficSplitEntry], revision_number: int, weight: int, port: int
) -> List[TrafficSplitEntry]:
    """
    Combines the previous traffic split rule with new rule.
    When filling the remaining weight, this function uses the one with higher revision number.

    For example, with the current rule of:
    - revision #2 (port 2222) 70%
    - revision #3 (port 3333) 30%

    with a call to this function with:
    - revision #4 (port 4444) 50%

    yields a new rule of:
    - revision #4 (port 4444) 50%
    - revision #3 (port 3333) 30%
    - revision #2 (port 2222) 20%

    Revision #3 takes priority over #2, because it has the higher number (3 > 2).
    """
    # Sort from latest revision (with highest number) to oldest
    rules_current = sorted(rules_current, key=lambda x: x.revision_number, reverse=True)

    rules_new: List[TrafficSplitEntry] = [
        TrafficSplitEntry(revision_number=revision_number, port=port, traffic_weight=weight)
    ]

    weight_remaining = 100 - weight

    # Iterate through current traffic rules and add them if possible
    for rule in rules_current:
        if weight_remaining <= 0:
            break
        new_weight = min(weight_remaining, rule.traffic_weight)
        rules_new.append(
            TrafficSplitEntry(
                revision_number=rule.revision_number, port=rule.port, traffic_weight=new_weight
            )
        )
        weight_remaining -= new_weight
        if weight_remaining <= 0:
            break

    if weight_remaining > 0:
        # This can happen if rules_current's weight do not sum up to 100
        # (this is possible for disabled gateways).
        # Handle this case safely by delegating all remaining weights to our target rule.
        rules_new[0].traffic_weight += weight_remaining

    return rules_new


def update_gateway_for_revision(
    organization: str,
    serving_name: str,
    revision_number: int,
    port: int,
    weight: int,
) -> ResponseModelServiceGatewayInfo:
    gateway_current = read_gateway(organization=organization, serving_name=serving_name)

    rules_new = _get_updated_traffic_split_rule(
        rules_current=gateway_current.rules or [],
        revision_number=revision_number,
        port=port,
        weight=weight,
    )

    gateway_updated = vessl_api.model_service_gateway_update_api(
        organization_name=organization,
        model_service_name=serving_name,
        model_service_gateway_update_api_input=ModelServiceGatewayUpdateAPIInput(
            enabled=True,
            endpoint=gateway_current.endpoint,
            ingress_class=gateway_current.ingress_class,
            annotations=gateway_current.annotations,
            traffic_split=rules_new,
        ),
    )
    return gateway_updated


def update_gateway_from_yaml(
    organization: str, serving_name: str, yaml_body: str
) -> ResponseModelServiceGatewayInfo:
    payload = ServingGatewayYamlImportAPIInput(yaml_body)

    return vessl_api.serving_gateway_yaml_import_api(
        organization_name=organization,
        model_service_name=serving_name,
        serving_gateway_yaml_import_api_input=payload,
    )
